# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-03-04 08:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('joyous', '0002_auto_20180228_1449'),
    ]

    operations = [
        migrations.AddField(
            model_name='extrainfopage',
            name='extra_title',
            field=models.CharField(blank=True, help_text='A more specific title for this occurence (optional)', max_length=255, verbose_name='Title'),
        ),
    ]
