// Generated by CoffeeScript 1.10.0
(function() {
  var time12hrFormat;

  time12hrFormat = "g:ia";

  $(function() {
    var ref;
    if ((((ref = $.datetimepicker) != null ? ref.setDateFormatter : void 0) != null) && (typeof moment !== "undefined" && moment !== null)) {
      time12hrFormat = "h:mma";
      $.datetimepicker.setDateFormatter("moment");
    }
  });

  this.initTime12hrChooser = function(id) {
    $('#' + id).datetimepicker({
      datepicker: false,
      formatTime: time12hrFormat,
      format: time12hrFormat,
      step: 30
    });
  };

}).call(this);
